/**
 * @file
 * Mapycz common behaviors.
 *
 * @codingStandardsIgnoreStart
 */

(function ($, window, Drupal) {
  'use strict';

  /**
   * @namespace
   */
  Drupal.mapycz = Drupal.mapycz || {};

  const translatedLayerNames = {
    basic: "základní",
    outdoor: "turistická",
    aerial: "letecká",
    winter: "zimní",
  };

  /**
   * Initiailizes map.
   */
  Drupal.mapycz.mapsInit = function (settings_, context) {
    // Settings.
    Drupal.mapycz.settings = {
      // Admin mode.
      admin: false,
      // Add suggestion textfield.
      suggest: false,
      // Auto compute center and zoom.
      computeCenterZoom: false,
      // Switchable layers.
      layerOptions: [],
    };
    for (var p in settings_) {
      Drupal.mapycz.settings[p] = settings_[p];
    }

    // If MapyCZ Loader is not present, lazy load it.
    if (typeof Loader === 'undefined') {
      $.getScript('https://api.mapy.cz/loader.js').done(function () {
        Drupal.mapycz._init(Drupal.mapycz.settings, context);
      });
    }
    else {
      Drupal.mapycz._init(Drupal.mapycz.settings, context);
    }
  };

  /**
   * Initialization callback from main init.
   */
  Drupal.mapycz._init = function (settings, context) {
    // Loader.
    Loader.async = true;
    Loader.load(null, { suggest: settings.suggest, poi: true }, function () {

      setTimeout(function () {
         Drupal.mapycz.loadLeafletMaps(settings);
      }, 200);
    });
  };

  function loadLeafletMapNameById(id) {
    var name = '';
    if (id == 1) {
      name = 'basic';
    }
    if (id == 2) {
      name = 'outdoor';
    }
    if (id == 3) {
      name = 'aerial';
    }
    if (id == 4) {
      name = 'winter';
    }

    return name;
  }
  function loadLeafletMapIdByName(name) {
    const nameToIdMap = {
      základní: 1,
      turistická: 2,
      letecká: 3,
      zimní: 4,
    };

    const id = nameToIdMap[name.toLowerCase()];
    return id || null;
  }

  function translateNameToCzech(englishName) {
    return translatedLayerNames[englishName] || englishName;
  }


  Drupal.mapycz.loadLeafletMaps = function (settings) {
    const API_KEY = drupalSettings.mapycz.api_key;
    // For mapyCz address field.
    var addressIds = [];
    $('.mapycz-address').each(function (idx, el) {
      addressIds.push(el.id);
    });
    $.each(addressIds, function (i, addressId) {
      var address_result = ''
      var $wrapper = $('#' + addressId + '-wrapper');
      if ($wrapper.hasClass('processed')) {
        return true;
      }
      $wrapper.addClass('processed');
      var lng = $wrapper.find('.mapycz-location .lng').html();
      var lat = $wrapper.find('.mapycz-location .lat').html();
      var geocodeUrl = `https://api.mapy.cz/v1/rgeocode?lat=${lat}&lon=${lng}&apikey=${API_KEY}`;

      // Perform the AJAX request to fetch the address.
      $.getJSON(geocodeUrl, function (data) {
        if (data && data.items && data.items.length > 0) {
          var address = data.items[0].name;
          address_result = `<span class="result-address">Adresa: ${address}</span>`;
          $wrapper.find('.mapycz-map-address').append(address_result);

        } else {
          console.log(`No address found for marker ${i}`);
        }

      }).fail(function (jqXHR, textStatus, errorThrown) {
        console.error(`Failed to fetch address for marker ${i}: ${textStatus}`, errorThrown);
      });
    });

    // For mapyCz map field.
    var leafletMapIds = [];
    $('.mapycz-map').each(function (idx, el) {
      leafletMapIds.push(el.id);
    });


    /*
    We create the map and set its initial coordinates and zoom.
    See https://leafletjs.com/reference.html#map
    */
    if (!window.mapyCzAdminMap) {
      window.mapyCzAdminMap = [];
    }

    $.each(leafletMapIds, function (i, mapyCzMapId) {
      // Get unprocessed wrapper.
      var $wrapper = $('#' + mapyCzMapId + '-wrapper');
      if ($wrapper.hasClass('processed')) {
        return true;
      }
      $wrapper.addClass('processed');
      let map = L.map(mapyCzMapId, {
        scrollWheelZoom: false // Disable mouse scroll zoom
      }).setView([50.725, 15.603], 15);
      var map_data = Drupal.mapycz.getMapData(mapyCzMapId);
      map.setView([map_data.center.lat, map_data.center.lng], map_data.zoom);


      if (settings.admin === true) {

        if (!window.mapyCzAdminMap[mapyCzMapId]) {
          window.mapyCzAdminMap[mapyCzMapId] = {};
          window.mapyCzAdminMap[mapyCzMapId]['markers'] = [];
          window.mapyCzAdminMap[mapyCzMapId]['map'] = map;
        }

        var $inputLat = $wrapper.find('.mapycz-input-location-lat');
        var $inputLng = $wrapper.find('.mapycz-input-location-lng');
        var $inputCenterLat = $wrapper.find('.mapycz-input-center-lat');
        var $inputCenterLng = $wrapper.find('.mapycz-input-center-lng');
        var $inputZoom = $wrapper.find('.mapycz-input-zoom');
        var $inputType = $wrapper.find('.mapycz-input-type');
        // Listen for click events on the map
        var currentMarker = null;
        if ($inputLat.val() && $inputLng.val()) {
            currentMarker = L.marker([$inputLat.val(), $inputLng.val()]).addTo(map);
        }
        window.mapyCzAdminMap[mapyCzMapId]['markers'] = [currentMarker];
        map.on('click', function(event) {
          if (!window.mapyCzAdminMap[mapyCzMapId]) {
            window.mapyCzAdminMap[mapyCzMapId] = map;
            window.mapyCzAdminMap[mapyCzMapId]['markers'] = [currentMarker];
          }
          window.mapyCzAdminMap[mapyCzMapId]['markers'].forEach(marker => currentMarker = marker);
          var lat = event.latlng.lat;
          var lng = event.latlng.lng;
          $inputLng.val(lng);
          $inputLat.val(lat);
          if (currentMarker) {
            map.removeLayer(currentMarker);
          }
          // Add a new marker at the clicked location
          currentMarker = L.marker([lat, lng]).addTo(map);
          window.mapyCzAdminMap[mapyCzMapId]['markers'] = [currentMarker];
        });

        map.on('zoomend', function() {
          $inputZoom.val(map.getZoom());
        });
        map.on('moveend', function() {
          var center = map.getCenter();
          $inputCenterLat.val(center.lat);
          $inputCenterLng.val(center.lng);
        });
        map.on('baselayerchange', function(event) {
          $inputType.val(loadLeafletMapIdByName(event.name));
        });
      }

      /*
      We store all our tile layers in an object, because we will
      need to pass that to the layers switching map control.
      */
      var tileLayers = {};
      var isConjoinedMap = false;
      if (map_data.markers.length > 1) {
        map_data.allowed_layers = [+map_data.type];
        isConjoinedMap = true;
      }
      var filteredLayers = map_data.all_layers;

      if (settings.admin === false && Object.keys(map_data.allowed_layers).length > 0) {
        filteredLayers = map_data.all_layers.filter(layer => layer in map_data.allowed_layers);

        if (!(map_data.type in map_data.allowed_layers)) {
          filteredLayers.push(map_data.type);
        }
      }

      $.each(filteredLayers, function (i, id) {
          let name = loadLeafletMapNameById(id);
          let czechName = translateNameToCzech(name);
          tileLayers[czechName] = L.tileLayer(`https://api.mapy.cz/v1/maptiles/${name}/256/{z}/{x}/{y}?apikey=${API_KEY}`, {
            minZoom: 0,
            maxZoom: 19,
            attribution: '<a href="https://api.mapy.cz/copyright" target="_blank">&copy; Seznam.cz a.s. a další</a>',
          });
        });

            /*
      Then we add the first raster tile layer to the map.
      See https://leafletjs.com/reference.html#tilelayer
      */
      if ((tileLayers && settings.admin === true && $inputType.val())) {
        let name = loadLeafletMapNameById($inputType.val());
        let czechName = translateNameToCzech(name);
        tileLayers[czechName].addTo(map);
      } else {
        if (!isConjoinedMap) {
          let basicLayer = translateNameToCzech('basic');
          tileLayers[basicLayer].addTo(map);
        }
      }

      if ((tileLayers && settings.admin === false && map_data.type)) {
        const mapName = loadLeafletMapNameById(map_data.type);
        const mapNameTranslated = translateNameToCzech(mapName);

        // Ensure the map name exists in tileLayers before adding it.
        if (mapNameTranslated && tileLayers[mapNameTranslated]) {
          tileLayers[mapNameTranslated].addTo(map);
        }
      }


      if (isConjoinedMap) {
        const c_id = +map_data.type;
        const c_name = loadLeafletMapNameById(c_id);
        const c_name_translated = translateNameToCzech(c_name);

        tileLayers[c_name_translated].addTo(map);
      }

      if (settings.layerOptions.length !== 0) {
        L.control.layers(tileLayers).addTo(map);
      }


      /*
      We also require you to include our logo somewhere over the map.
      We create our own map control implementing a documented interface,
      that shows a clickable logo.
      See https://leafletjs.com/reference.html#control
      */
      const LogoControl = L.Control.extend({
        options: {
          position: 'bottomleft',
        },

        onAdd: function (map) {
          const container = L.DomUtil.create('div');
          const link = L.DomUtil.create('a', '', container);

          link.setAttribute('href', 'http://mapy.cz/');
          link.setAttribute('target', '_blank');
          link.innerHTML = '<img src="https://api.mapy.cz/img/api/logo.svg" />';
          L.DomEvent.disableClickPropagation(link);

          return container;
        },
      });

      if (settings.admin === false) {
        $.each(map_data.markers, function (i, marker) {
          var leafletMarker = L.marker([marker.lat, marker.lng]).addTo(map);

          if (marker.title) {
            var markerPopupContent = marker.title;

            // Reverse geocode if print_address is true
            if (marker.print_address) {
              var geocodeUrl = `https://api.mapy.cz/v1/rgeocode?lat=${marker.lat}&lon=${marker.lng}&apikey=${API_KEY}`;

              // Perform the AJAX request to fetch the address
              $.getJSON(geocodeUrl, function (data) {
                if (data && data.items && data.items.length > 0) {
                  var address = data.items[0].name;
                  markerPopupContent += `<br><span class="result-address"><i>Adresa: ${address}</i></span>`;
                } else {
                  console.log(`No address found for marker ${i}`);
                }

                // Bind the updated popup content to the marker AFTER the address is fetched
                leafletMarker.bindPopup(markerPopupContent);
              }).fail(function (jqXHR, textStatus, errorThrown) {
                console.error(`Failed to fetch address for marker ${i}: ${textStatus}`, errorThrown);
                // Fallback to binding popup without address if the request fails
                leafletMarker.bindPopup(markerPopupContent);
              });
            } else {
              // If no address is to be printed, bind the popup immediately
              leafletMarker.bindPopup(markerPopupContent);
            }
          }

        });

        // Center around the markers if multiple present.
        var bounds = [];
        $.each(map_data.markers, function (i, marker) {
          bounds.push([marker.lat, marker.lng]);
        });

        if (bounds.length > 1) {
          map.fitBounds(bounds, {
            padding: [50, 50],  // Adjust padding to avoid markers being too close to the map's edges
            maxZoom: 10         // Set a maximum zoom level to avoid excessive zooming
          });
        }

        // Add layout switcher if any.
        if (map_data.show_layout_switcher == 1) {
          L.control.layers(tileLayers).addTo(map);
        }
      }


      // finally we add our LogoControl to the map
      new LogoControl().addTo(map);
    });


  };


  /**
   * Parses map's data from HTML structure.
   *
   * @param mapId
   *   Id of map - div with mapycz-map class.
   *
   * @returns
   *   Data object.
   */
  Drupal.mapycz.getMapData = function (mapId) {
    var $wrapper = $('#' + mapId + '-wrapper');
    var data = {};
    data.type = $wrapper.find('.mapycz-settings .mapycz-type').html();
    if (!data.type) {
      data.type = 1;
    }
    data.zoom = parseInt($wrapper.find('.mapycz-settings .mapycz-zoom').html());
    data.center = {};
    data.center.lng = $wrapper.find('.mapycz-settings .mapycz-center .lng').html();
    data.center.lat = $wrapper.find('.mapycz-settings .mapycz-center .lat').html();
    data.show_poi = parseInt($wrapper.find('.mapycz-settings .mapycz-poi').html());
    data.show_layout_switcher = parseInt($wrapper.find('.mapycz-settings .mapycz-layer-switcher').html());
    data.allowed_layers = $wrapper.find('.mapycz-settings .mapycz-type').data('allowed-layers');
    data.all_layers = [1, 2, 3, 4];
    data.markers = [];
    $wrapper.find('.mapycz-location').each(function (i, location) {
      var marker = {};
      marker.lat = $(location).find('.lat').html();
      marker.lng = $(location).find('.lng').html();
      marker.title = $(location).find('.marker-title').html();
      marker.print_address = $wrapper.find('.marker-address').html();
      data.markers.push(marker);
    });
    return data;
  };
})(jQuery, window, Drupal);
